/*
 * Copyright (C) 2019 Rick Busarow
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.rickbusarow.dispatcherprovider

import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Job
import kotlinx.coroutines.SupervisorJob

interface IOCoroutineScope : CoroutineScope
interface DefaultCoroutineScope : CoroutineScope
interface MainCoroutineScope : CoroutineScope
interface MainImmediateCoroutineScope : CoroutineScope
interface UnconfinedCoroutineScope : CoroutineScope

fun DefaultCoroutineScope(
  job: Job = SupervisorJob(),
  dispatcherProvider: DispatcherProvider = DefaultDispatcherProvider()
): DefaultCoroutineScope = object : DefaultCoroutineScope {
  override val coroutineContext = job + dispatcherProvider.default + dispatcherProvider
}

fun IOCoroutineScope(
  job: Job = SupervisorJob(),
  dispatcherProvider: DispatcherProvider = DefaultDispatcherProvider()
): IOCoroutineScope = object : IOCoroutineScope {
  override val coroutineContext = job + dispatcherProvider.io + dispatcherProvider
}

fun MainCoroutineScope(
  job: Job = SupervisorJob(),
  dispatcherProvider: DispatcherProvider = DefaultDispatcherProvider()
): MainCoroutineScope = object : MainCoroutineScope {
  override val coroutineContext = job + dispatcherProvider.main + dispatcherProvider
}

fun MainImmediateCoroutineScope(
  job: Job = SupervisorJob(), dispatcherProvider:
  DispatcherProvider = DefaultDispatcherProvider()
): MainImmediateCoroutineScope = object :
  MainImmediateCoroutineScope {
  override val coroutineContext = job + dispatcherProvider.mainImmediate +
      dispatcherProvider
}

fun UnconfinedCoroutineScope(
  job: Job = SupervisorJob(),
  dispatcherProvider: DispatcherProvider = DefaultDispatcherProvider()
): UnconfinedCoroutineScope = object : UnconfinedCoroutineScope {
  override val coroutineContext = job + dispatcherProvider.unconfined + dispatcherProvider
}


